import type { Patch } from 'rko'
import { TLDR } from '~state/tldr'
import type { TLDrawShape, Data, TLDrawCommand } from '~types'

export function create(data: Data, shapes: TLDrawShape[]): TLDrawCommand {
  const { currentPageId } = data.appState

  const beforeShapes: Record<string, Patch<TLDrawShape> | null> = {}
  const afterShapes: Record<string, Patch<TLDrawShape> | null> = {}

  shapes.forEach((shape) => {
    beforeShapes[shape.id] = null
    afterShapes[shape.id] = shape
  })

  return {
    id: 'create',
    before: {
      document: {
        pages: {
          [currentPageId]: {
            shapes: beforeShapes,
          },
        },
        pageStates: {
          [currentPageId]: {
            selectedIds: [...TLDR.getSelectedIds(data, currentPageId)],
          },
        },
      },
    },
    after: {
      document: {
        pages: {
          [currentPageId]: {
            shapes: afterShapes,
          },
        },
        pageStates: {
          [currentPageId]: {
            selectedIds: shapes.map((shape) => shape.id),
          },
        },
      },
    },
  }
}
